
## Rack::Radar

**A simple API for testing Rack applications via Rack::MockRequest**<br>
*inspired by [rack-test](https://github.com/brynary/rack-test)*


## Installation

Add this line to your application's Gemfile:

```ruby
gem 'rack-radar'
```

And then execute:

```bash
$ bundle
```

Or install it yourself as:

```bash
$ gem install rack-radar
```

then load it using `require 'rack-radar'`


## Use

Simply `include Rack::Radar` in your tests.

## App

Call `app RackApp` inside testing suite to set app to be tested.

**Minitest Example:**

```ruby
require 'rack-radar'

class MyTests < MiniTest::Unit::TestCase

  include Rack::Radar

  def setup
    app MyRackApp
  end

  def test
    get '/url'
    assert_equal last_response.status, 200
  end
end
```

Multiple apps can be tested within same suite.<br/>
Each app will run own session.

**Minitest Example:**

```ruby
require 'rack-radar'

class MyTests < MiniTest::Unit::TestCase

  include Rack::Radar

  def setup
    app MyRackApp
  end

  def test
    # querying default app
    get '/url'
    assert_equal last_response.status, 200

    # testing ForumApp
    app ForumApp
    get '/posts'
    assert_equal last_response.status, 200

    # back to default app
    app MyRackApp
    get '/url'
    assert_equal last_response.status, 200
  end
end
```


## Resetting App

Sometimes you need to start over with a new app in pristine state, i.e. no cookies, no headers etc.

To achieve this, simply call `reset_app!` (or `reset_browser!`).

## Requests

Use one of `get`, `post`, `put`, `patch`, `delete`, `options`, `head`
to make requests via Rack::Radar

To make a secure request, add `s_` prefix:

```ruby
s_get  '/path'
s_post '/path'
# etc.
```

To make a request via XHR, aka Ajax, add `_x` suffix:

```ruby
get_x  '/path'
post_x '/path'
# etc.
```

To make a secure request via XHR, add both `s_` and `_x`:

```ruby
s_get_x  '/path'
s_post_x '/path'
# etc.
```

In terms of arguments, making HTTP requests via Rack::Radar is identical to calling regular Ruby methods.<br/>
That's it, you do not need to join parameters into a string.<br/>
Just pass them as usual arguments:

```ruby
post '/news', :create, :title => rand
post '/news', :update, id, :title => rand
get  '/news', :delete, id
```

## Map

Previous example works just fine, however it is redundant and inconsistent.<br/>
Just imagine that tested app changed its base URL from /news to /headlines.

The solution is simple.<br/>
Use `map` to define a base URL that will be prepended to each request,<br/>
except ones starting with a slash or a protocol(http://, https:// etc.) of course.

```ruby
app MyRackApp
map '/news'

post :create, :title => rand
post :update, id, :title => rand
get  :delete, id
```

**Note:** requests starting with a slash or protocol(http://, https:// etc.)
wont use base URL defined by `map`.<br/>

**Note:** when you switching tested app, make sure you also change the map.

To disable mapping, simply call `map nil`

## Cookies

**Set cookies:**

```ruby
cookies['name'] = 'value'
```

**Read cookies:**

```ruby
cookie = cookies['name']
```

**Delete a cookie:**

```ruby
cookies.delete 'name'
```


**Clear all cookies:**

```ruby
cookies.clear
```

Each app uses its own cookies jar.

## Headers

Rack::Radar allow to set headers that will be sent to app on all consequent requests.

**Set headers:**

```ruby
header['User-Agent']   = 'Rack::Radar'
header['Content-Type'] = 'text/plain'
header['rack.input']   = 'someString'
# etc.
```

**Read headers:**

```ruby
header = headers['User-Agent']
# etc.
```

**Delete a header:**

```ruby
headers.delete 'User-Agent'
```

**Clear all headers:**

```ruby
headers.clear
```

Each app uses its own headers.

## Authorization

**Basic Auth:**

```ruby
authorize 'user', 'pass'
```

**Reset earlier set Basic authorization header:**

```ruby
reset_basic_auth!
```

**Digest Auth:**

```ruby
digest_authorize 'user', 'pass'
```

**Reset earlier set Digest authorization header:**

```ruby
reset_digest_auth!
```

**Reset ANY earlier set authorization header:**

```ruby
reset_auth!
```

## Follow Redirects

By default, Rack::Radar wont follow redirects.

If last response is a redirect and you want Rack::Radar to follow it, use `follow_redirect!`


## Contributors

Want to contribute? Great! Contributors highly wanted and welcome!

1. Fork it
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request


## License

Copyright &copy; 2014 Slee Woo &lt;mail@sleewoo.com&gt;

Distributed under the **[MIT License](https://github.com/sleewoo/rack-radar/blob/master/LICENSE)**


