require 'test_setup'

class CookiesTest
  include Minispec

  class App < Air

    def index names
      cookies.values_at(*names.split(',')).inspect
    end

    def post_index
      post_params.each_pair do |n, v|
        v.is_a?(Hash) &&
          (expires = v['expires']) &&
          (v['expires'] = ::Time.parse(expires))
        response.set_cookie n, v
      end
    end

    def delete_index names
      names.split(',').each do |n|
        response.delete_cookie n, params[n] || {}
      end
    end

    def post_deeper__path
      post_params.each_pair { |n, v| response.set_cookie n, v }
    end

    def deeper name
      cookies[name]
    end
  end

  helper :persisted? do
    assert(cookies[@var]) == @val
  end

  helper :not_persisted do
    is(cookies[@var]).nil?
  end

  helper :disposed? do
    assert(last_response.body) == @expected
  end

  helper :not_disposed do
    assert(last_response.body) == @expected_nil
  end

  before_all { app(App) }
  before { cookies.clear }

  context 'set/get via HTTP' do

    before do
      @var, @val = 2.times.map { 5.times.map { (('a'..'z').to_a + ('A'..'Z').to_a + (1..50).to_a)[rand(100)] }.join }
      @expected, @expected_nil = [@val].inspect, [nil].inspect
    end

    should 'persist/dispose cause by default cookie`s path is set to current URI path' do
      post @var => @val
      is.persisted?

      get @var
      is.disposed?
    end

    should 'persist/dispose cause given path matches default cookie`s path' do
      post @var => {:value => @val, :path => '/'}
      is.persisted?

      get @var
      is.disposed?
    end

    should 'NOT persist/dispose cause wrong path provided - /blah is not a prefix of /' do
      post @var => {:value => @val, :path => '/blah'}
      assure.not_persisted

      get @var
      assure.not_disposed
    end

    should 'persist/dispose cause /deeper is a prefix of /deeper/path' do
      post '/deeper/path', @var => @val
      is.persisted?

      get '/deeper', @var
      assert(last_response.body) == @val
    end

    should 'persist but not dispose cause path set automatically to /deeper, and it is not a prefix for /' do
      post '/deeper/path', @var => @val
      is.persisted?

      get @var
      assure.not_disposed
    end

    should 'persist and dispose for /deeper path' do
      post '/deeper/path', @var => @val
      is.persisted?

      get '/deeper', @var
      assert(last_response.body) == @val
    end

    should 'persist/dispose cause path set to /' do
      post '/deeper/path', @var => {:value => @val, :path => '/'}
      is.persisted?

      get @var
      is.disposed?
    end

    should 'NOT dispose - expires is in past' do
      post @var => {:value => @val, :expires => ::Rack::Utils.rfc2822(::Time.now.gmtime - 1)}
      is.persisted?

      get @var
      assure.not_disposed
    end

    should 'NOT persist/dispose cause inner domain given' do
      post @var => {:value => @val, :domain => 'x.org'}
      assure.not_persisted

      get @var
      assure.not_disposed
    end

    should 'persist/dispose when domains matches' do
      header['HTTP_HOST'] = 'some.tld'
      post 'http://' + header['HTTP_HOST'], @var => @val
      get  'http://' + header['HTTP_HOST'], @var
      is.disposed?
      headers.delete 'HTTP_HOST'
    end

    should 'prefer more specific cookies' do
      domain = 'http://foo.bar'
      subdomain = 'http://a.foo.bar'

      header['HTTP_HOST'] = 'a.foo.bar'
      post subdomain, 'var' => subdomain

      header['HTTP_HOST'] = 'foo.bar'
      post domain, 'var' => domain

      get domain, 'var'
      is(last_response.body) == [domain].inspect

      header['HTTP_HOST'] = 'a.foo.bar'
      get subdomain, 'var'
      is(last_response.body) == [subdomain].inspect
      headers.delete 'HTTP_HOST'
    end

    should 'NOT persist/dispose cause secure cookie are accessed via un-secure connection' do
      post @var => {:value => @val, :secure => 'true'}
      is(cookies[@var]).nil?

      get @var
      assure.not_disposed
    end

    should 'persist/dispose cause secure cookie are accessed via secure connection' do
      s_post @var => {:value => @val, :secure => 'true'}
      is(cookies[@var]) == @val

      s_get @var
      expect(last_response.body) == @expected
    end

    should 'delete a cookie' do

      post @var => @val
      get @var
      is.persisted?

      delete @var
      get @var
      assure.not_disposed
    end

    should 'set/get multiple cookies at once' do
      vars, vals = 2.times.map { 5.times.map { 5.times.map { ('a'..'z').to_a[rand(26)] }.join } }
      params = Hash[vars.zip vals]
      post params
      get vars.join(',')
      assert(last_response.body) == params.values_at(*vars).inspect
    end
  end

  context 'jars are domain specific' do

    should 'use separate jar for each domain' do

      header['HTTP_HOST'] = 'foo.bar'
      domain = 'http://' + header['HTTP_HOST']
      post domain, 'var' => domain
      get domain, 'var'
      is(last_response.body) == [domain].inspect
      headers.delete 'HTTP_HOST'

      header['HTTP_HOST'] = 'bar.foo'
      domain = 'http://' + header['HTTP_HOST']
      post domain, 'var' => domain
      get domain, 'var'
      is(last_response.body) == [domain].inspect
      headers.delete 'HTTP_HOST'

      header['HTTP_HOST'] = 'foo.bar'
      domain = 'http://' + header['HTTP_HOST']
      get domain, 'var'
      is(last_response.body) == [domain].inspect
      headers.delete 'HTTP_HOST'
    end

    should 'use same jar for a domain and its subdomains' do
      %w[a  b  c  a.b  a.b.c].each do |subdomain|
        header['HTTP_HOST'] = '%s.foo.bar' % subdomain
        domain = 'http://' + header['HTTP_HOST']
        get domain, 'var'
        is(last_response.body) == ['http://foo.bar'].inspect
        headers.delete 'HTTP_HOST'
      end
    end

    should 'domains names are case insensitive' do
      %w[BAR.foo  bar.FOO   BAR.FOO].each do |host|
        header['HTTP_HOST'] = host
        domain = 'http://' + header['HTTP_HOST']
        get domain, 'var'
        is(last_response.body) == ['http://bar.foo'].inspect
        headers.delete 'HTTP_HOST'
      end
    end

  end

  context 'set/get directly' do

    before_all { app(App) }

    should 'set/get cookies properly' do
      n, size = 10, cookies.size
      n.times do
        var, val = 2.times.map { 5.times.map { ('a'..'z').to_a[rand(26)] }.join }
        cookies[var] = val
        expect(cookies[var]) == val
      end
      expect(cookies.size) == size + n
    end

    should 'delete a cookie' do

      cookies['foo'] = 'bar'
      expect(cookies['foo']) == 'bar'

      cookies.delete 'foo'
      is(cookies['foo']).nil?
    end

    should 'clear cookies' do
      cookies.clear
      expect(cookies.size) == 0
    end

    should 'escape values' do
      cookies['var'] = 'foo;bar'
      expect(cookies['var']) == 'foo;bar'

      cookies['var'] = ';bar;foo;'
      expect(cookies['var']) == ';bar;foo;'
    end
  end
end
